
#ifndef MAIN_C
#define MAIN_C

/** COMPILER AND LIBRARY INCLUDES *******************************************************/
#include "./USB/usb.h"
#include "HardwareProfile.h"

#include "./USB/usb_function_hid.h"
#include "plib.h" 
#include "math.h"
#include "Wireless.h"

/** CONFIGURATION **************************************************/
     
#pragma config UPLLEN   = ON            // USB PLL Enabled
#pragma config FPLLMUL  = MUL_18        // PLL Multiplier Test 4MHzx18=72MHz CPU
#pragma config UPLLIDIV = DIV_2         // USB PLL Input Divider
#pragma config FPLLIDIV = DIV_2         // PLL Input Divider
#pragma config FPLLODIV = DIV_1         // PLL Output Divider
#pragma config FPBDIV   = DIV_1         // Peripheral Clock divisor -> Full Speed 72MHz
//#pragma config FPBDIV   = DIV_2         // Peripheral Clock divisor Test 36MHz Peripeheral Clock 
#pragma config FWDTEN   = OFF           // Watchdog Timer 
#pragma config WDTPS    = PS1           // Watchdog Timer Postscale
#pragma config FCKSM    = CSDCMD        // Clock Switching & Fail Safe Clock Monitor
#pragma config OSCIOFNC = OFF           // CLKO Enable
#pragma config POSCMOD  = HS            // Primary Oscillator
#pragma config IESO     = OFF           // Internal/External Switch-over
#pragma config FSOSCEN  = OFF           // Secondary Oscillator Enable
#pragma config FNOSC    = PRIPLL        // Oscillator Selection
#pragma config CP       = OFF           // Code Protect
#pragma config BWP      = OFF           // Boot Flash Write Protect
#pragma config PWP      = OFF           // Program Flash Write Protect
//#pragma config ICESEL   = ICS_PGx2      // ICE/ICD Comm Channel Select
#pragma config DEBUG    = OFF           // Debugger Disabled for Starter Kit
       

#define SYS_FREQ		(72000000)
#define PER_FREQ		(72000000)


/** PROJECT INCLUDES *******************************************************/
#include "MPU6050.h"
#include "I2C.h"




/** USB Communication ******************************************************/
//USB Variables and System 
#pragma udata
unsigned char ReceivedDataBuffer[64];
unsigned char ToSendDataBuffer[64];
USB_HANDLE USBOutHandle = 0;
USB_HANDLE USBInHandle = 0;
BOOL blinkStatusValid = TRUE;

void BlinkUSBStatus(void);
void ProcessIO(void);
void USBCBSendResume(void);


/* Motion Sensor Data Storage */
/*Attention 
char - 8bit
short - 16bit
int - 32bit
long - 64bit
float - 32bit
double - 64bit
*/
signed short ACCEL_XOUT;
signed short ACCEL_YOUT;
signed short ACCEL_ZOUT;
float GYRO_XRATE;
float GYRO_YRATE;
 float GYRO_ZRATE;
	int GYRO_XRATERAW;
int GYRO_YRATERAW;
	 int GYRO_ZRATERAW;
	
	 unsigned char GYRO_XOUT_L;
	 unsigned char GYRO_XOUT_H;
	 unsigned char GYRO_YOUT_L;
	 unsigned char GYRO_YOUT_H;
	 unsigned char GYRO_ZOUT_L;
	 unsigned char GYRO_ZOUT_H;
	 signed int GYRO_XOUT;
	 signed int GYRO_YOUT;
	 signed int GYRO_ZOUT;	
	
	 unsigned char ACCEL_XOUT_L;
	 unsigned char ACCEL_XOUT_H;
	 unsigned char ACCEL_YOUT_L;
	 unsigned char ACCEL_YOUT_H;
	 unsigned char ACCEL_ZOUT_L;
	 unsigned char ACCEL_ZOUT_H;
	
	 signed long GYRO_XOUT_OFFSET_1000SUM;
	 signed long GYRO_YOUT_OFFSET_1000SUM;
	 signed long GYRO_ZOUT_OFFSET_1000SUM;
	
	 float GYRO_XANGLE;
	 float GYRO_YANGLE;
	 float GYRO_ZANGLE;
	 long GYRO_XANGLERAW;
	 long GYRO_YANGLERAW;
	 long GYRO_ZANGLERAW;
	 float ACCEL_XANGLE;
	 float ACCEL_YANGLE;
	 float ACCEL_ZANGLE;
	
	 float KALMAN_XANGLE;
	 float KALMAN_YANGLE;
	 float KALMAN_ZANGLE;
	
	 signed int GYRO_XOUT_OFFSET;
	 signed int GYRO_YOUT_OFFSET;
	 signed int GYRO_ZOUT_OFFSET;

	float COMPLEMENTARY_XANGLE, COMPLEMENTARY_YANGLE;

unsigned char MAGENT_XOUT_H;
 unsigned char MAGENT_XOUT_L;
 unsigned char MAGENT_YOUT_H;
 unsigned char MAGENT_YOUT_L;
 unsigned char MAGENT_ZOUT_H;
 unsigned char MAGENT_ZOUT_L;
 signed int MAGENT_XOUT;
 signed int	MAGENT_YOUT;
 signed int	MAGENT_ZOUT;



// Redirecting printf to datafield 

typedef union {
unsigned char u_value;
char s_value;
} char_signed_unsigned;

#define MAX_Message 16600

char_signed_unsigned MessageBuffer[MAX_Message+2];
unsigned int MessageCounter=0;


void _mon_putc(char c)
{
    //myOutputFunction(c);    // printf() passes each character to write to _mon_putc(), which in turn passes each character to a custom output function
	MessageBuffer[MessageCounter].s_value=c;
	MessageCounter++;
	if (MessageCounter>MAX_Message) MessageCounter=0;
}











/** FIFO Buffer for communication and Packages for Data Transfer *******************************/
/*FIFO for communication*/
typedef union {
	unsigned char value;
  struct {
    unsigned SECSYNC:1;
    unsigned TRIGSYNC:1;
    unsigned Trigger1stat:1;
    unsigned Trigger2stat:1;
	unsigned SystemStat:1;
	unsigned BufferOverFlow:1;
	unsigned ExperimentRunning:1;
	unsigned CycleTorque:1;
  };
  
} StatusType;

typedef struct {
WORD_VAL ADCVal;
WORD_VAL time;
StatusType status;
} STATE_PACKET;


typedef union {
unsigned int value;
struct {
	unsigned char byte1;
	unsigned char byte2;
	unsigned char byte3;
	unsigned char byte4;
		};
} LongDataType;


typedef union {
float value;
struct {
	unsigned char byte1;
	unsigned char byte2;
	unsigned char byte3;
	unsigned char byte4;
		};
} FloatDataType;


/* Time and Sync Definition */

#define EVENT_SECSYNC 4
#define EVENT_EGDETRIGGER2 5
#define EVENT_EGDETRIGGER1 7
#define EVENT_ENDOFEXPERIMENT 8

#define MAX_BUFFER 200
#pragma udata buffer_data1
WORD_VAL TimeStamp[MAX_BUFFER];
#pragma udata

#pragma udata buffer_data2
WORD_VAL ADCVal[MAX_BUFFER];
#pragma udata

#pragma udata buffer_data3
unsigned char Status[MAX_BUFFER];
#pragma udata

#pragma udata general
/** FIFO Controll */
unsigned short WCounter;
unsigned short RCounter;
unsigned int TimeOfLastStamp;



/** Controll of ADConverter and Relevant Values *******************************/
//ADC Controll 
#define ONE_VOLT 310
#define HALF_VOLT 155
#define QUARTER_VOLT 77

unsigned int  LEVEL_1 = ONE_VOLT; 
unsigned int  LEVEL_2 = 3*ONE_VOLT;

//Storing ADC Values 
unsigned int PressureFeedbackADC;
unsigned int LightBarrierADC;
WORD_VAL LightBarrierAnalogVal;
WORD_VAL PressureFeedbackADCVal;
WORD_VAL oldADCVal;

//Reading and writing ADC
void InitADC( void );
void ServiceADC( void );

/********* Mode ****************/
unsigned char Mode = 0; 
//Mode =0  IDLE
//Mode > 0  Experiment
//Mode 1 Fixed Torque
//Mode 2 Fixed RPM
//Mode 3 Modulated Torque

/** Engine Controll *******************************/
WORD_VAL RPM;
int SetEngine(unsigned char power);
unsigned char GetEnginePower();
#define VALVEPower LATE

unsigned char IDLE_Engine_Power;

/******* Trigger Controll *******************/
//LATG8 is Trigger Channel 1
//LATG9 is Trigger Channel 2

int dummy;

#define TRIGGER_1              dummy

#define TRIGGER_2              LATGbits.LATG9

#define TRIGGER_3				LATGbits.LATG8	

/********** Time Control ********************/
//Attention the peripheral clock divisor is deactivated if optimal system settings are used!
//72 000 000 / 256 = 281250
#define one_second 281250
#define half_second 140625
#define one_second_f 281250.0
#define hundred_ms 28125
#define two_ms 562
#define one_ms 282
#define half_ms 141


/** PRIVATE PROTOTYPES *********************************************/

//DEbug Data
BYTE  txBuffer[] = {20, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19};
BYTE  rxBuffer[61];    // Length byte  + 2 status bytes are not stored in this buffer



/** DECLARATIONS ***************************************************/
#pragma code

/********************************************************************
 * Function:        void main(void)
 *******************************************************************/
//Time 	
unsigned int time;

LongDataType ContinousTime;




void __ISR( _TIMER_2_VECTOR ,ipl7) T2Interrupt(void)
{
	//Called every 15ms

	
	

	//TRIGGER_3=0;
	mT2ClearIntFlag();
};


#define LED1 LATBbits.LATB15
#define LED2 LATBbits.LATB12

//* Setup Gyro *///

void Zero_Sensors()
{
	
	float BUFFER_XANGLE = 0;
	float BUFFER_YANGLE = 0;
	int x = 0;
	for(x=0; x<100; x++)
	{
		Get_Accel_Values();
		Get_Accel_Angles();
		BUFFER_XANGLE += ACCEL_XANGLE;
		BUFFER_YANGLE += ACCEL_YANGLE;
		__delay_ms(1);
	}
	COMPLEMENTARY_XANGLE = BUFFER_XANGLE/100.0;
	COMPLEMENTARY_YANGLE = BUFFER_YANGLE/100.0;
	GYRO_XANGLE = BUFFER_XANGLE/100.0;
	GYRO_YANGLE = BUFFER_YANGLE/100.0;
}	

int InitAndControllSensor(void)
{	int counter=0;
	int error = 1;
	int trial=0;
	while (trial<4)
	{
	trial=trial+1;
	Setup_I2C();
	LED1=1;
	if (MPU6050_Test_I2C()) 
	{	
	printf("MPU Test I2C Failed \n");
	}
	else trial=5;
	};

	while(error==1 && counter<10)
	{
		counter++;
		Setup_MPU6050();
		LED2=1;
		if (MPU6050_Test_I2C()==0)
		{ 
		error = MPU6050_Check_Registers();
		};		
	}
	if (counter==10) printf("Start failed Counter is %d\n",counter);
	Calibrate_Gyros(); //switch on for real use
	Zero_Sensors();
	return error;
};	
	

unsigned int UIntABS(unsigned int v1, unsigned int v2)
{
if (v1>v2) return (v1-v2);
return v2-v1;
}




int main(void)
{   
	unsigned int ADCdiff=0;
	unsigned int packagenr=0;   
	long last_sec_sync_time=0;
	long seconds=0;
	int errorcode;
	unsigned int counter=0;
	unsigned char Data = 0x00;
	unsigned char Failed = 0;
	UINT8 length;

	//Optimize Systemperformance
//	SYSTEMConfigPerformance(SYS_FREQ);

	//Free POrt for IO Pin 
	DDPCONbits.JTAGEN=0;	

	//Configure LEDs
	PORTSetPinsDigitalOut(IOPORT_B, BIT_15 | BIT_12 );
		
	//Init Global Time
	OpenTimer45(T45_ON | T45_PS_1_256 | T4_32BIT_MODE_ON, 0xFFFFFFFF ); // 36 000 000 Hz Peripheral Bus / 256 =140625 Hz  
	WriteTimer45(0); 
	
	//Init USB
	USBOutHandle = 0;
    USBInHandle = 0;
    USBDeviceInit();
    USBDeviceAttach();

	//Init Interrupt 15ms
	OpenTimer2(T2_ON | T1_PS_1_256  , 140 ); //140 is 15 micro seconds
	mT2SetIntPriority(7);	
	mT2ClearIntFlag();
	mT2IntEnable( 1 );
	
	//Now System is ready get Motion Sensor Ready
	//InitAndControllSensor();
	LED1=0;
	LED2=0;
	printf("Start the system\n");
	
/////////////////////////////Debug Funkmodul

SetUpSystem();
//Data=0;
//printf("\n SetUp Funk Modul done0, was 0x%x", Data);
printf("Activation of Funk Module OK\n");
LED1=1;
LED2=1;

if (1) {
BYTE addr=0x0E;
unsigned char value=0;
unsigned char value2=0;
//halSpiWriteReg(addr, value);
value=halSpiReadReg(addr);
printf("\n Value 0x0E=%x  should be 1D\n ", value);
};

while(0) // 1:Receive Module 0:transmit 
{
	ProcessIO();
	//Calculate Time 
	time=ReadTimer45();

if (UIntABS(time,last_sec_sync_time)>one_second )
	{
	
		last_sec_sync_time=time;
		seconds++;
		LED2=!LED2;
	};

//	halRfSendPacket(txBuffer, sizeof(txBuffer));
	halRfReceivePacket(rxBuffer, &length);
	

	ACCEL_XOUT_L=rxBuffer[0];
	ACCEL_XOUT_H=rxBuffer[1];;
 	ACCEL_YOUT_L=rxBuffer[2];;
	ACCEL_YOUT_H=rxBuffer[3];;
	ACCEL_ZOUT_L=rxBuffer[4];;
	ACCEL_ZOUT_H=rxBuffer[5];;

	printf("%d\t %d\t %d\n", rxBuffer[0], rxBuffer[1], rxBuffer[2]);	
};

//////////////////////////End Debug 




	printf("Now Activate Sensors\n");	
	errorcode=InitAndControllSensor();
	LDByteWriteI2C(Mag_AK8975C_ADDRESS,0x0a, 0x01);
	LDByteReadI2C(Mag_AK8975C_ADDRESS, 0x0a, &Data, 1);

	txBuffer[1]=7;txBuffer[2]=7;txBuffer[3]=Data;
	halRfSendPacket(txBuffer, sizeof(txBuffer));

	if(Data != 0x01) 
		{ printf("\nRegister check 1 failed, value should be 0x01, was 0x%x", Data); Failed = 1; }
	LDByteReadI2C(Mag_AK8975C_ADDRESS, 0x0, &Data, 1);
	printf("\nMagnetometer check Register 0 return  value was 0x%x\n", Data);  

	if (errorcode==0) 
		{
		txBuffer[1]=7;txBuffer[2]=7;txBuffer[3]=123;
		halRfSendPacket(txBuffer, sizeof(txBuffer));
		printf("Sensor Activated\n");} 
		else 
		{
			txBuffer[1]=7;txBuffer[2]=0;txBuffer[3]=0;
		halRfSendPacket(txBuffer, sizeof(txBuffer));
		printf("Sensor Activation Failed Errorcode: %d\n",errorcode);
		}
	
	LED1=1;
	LED2=0;		

	txBuffer[1]=7;txBuffer[2]=7;txBuffer[3]=7;
	halRfSendPacket(txBuffer, sizeof(txBuffer));



    while(1) //Send Module 
    {
        			 
    ProcessIO(); // Controll of the systems mode and feedback of current RPM Speed
	//Get Values from Motion Sensor 
	Get_Accel_Values();
	Get_Accel_Angles();
	Get_Gyro_Rates();

	//Calculate Time 
	time=ReadTimer45();

	if (UIntABS(time,last_sec_sync_time)>hundred_ms )
	{
		Get_Accel_Values();
		Get_Accel_Angles();
		Get_Gyro_Rates();
		Get_Magnetometer_Values();
		LDByteWriteI2C(Mag_AK8975C_ADDRESS,0x0a, 0x01);
		//printf("%f\t %f\t %f\n", ACCEL_XANGLE, ACCEL_YANGLE, ACCEL_ZANGLE);
		//printf("%d\t %d\t %d\n", MAGENT_XOUT, MAGENT_YOUT, MAGENT_ZOUT);
		//LDByteWriteI2C(Mag_AK8975C_ADDRESS,0x0a, 0x01);
	

	};

	if (UIntABS(time,last_sec_sync_time)>3*hundred_ms )
	{
	
		last_sec_sync_time=time;
		seconds++;
		LED2=!LED2;
	 
	 txBuffer[1]=ACCEL_XOUT_L;
	 txBuffer[2]=ACCEL_XOUT_H;
	 txBuffer[3]=ACCEL_YOUT_L;
	 txBuffer[4]=ACCEL_YOUT_H;
	 txBuffer[5]=ACCEL_ZOUT_L;
	 txBuffer[6]=ACCEL_ZOUT_H;

	halRfSendPacket(txBuffer, sizeof(txBuffer));
	};
	
	
	
	}//end while
	return 0;
}//end main

















/********************************************************************
 * Function:        SetEngine
 * Overview:        Uniuque Controll of the Engines Torque
 *
 * Note:            None
 *******************************************************************/

int SetEngine(unsigned char power)
{
	VALVEPower=power;
};

unsigned char GetEnginePower()
{
	return VALVEPower;
};








/********************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user
 *                  routines. It is a mixture of both USB and
 *                  non-USB tasks.
 *
 * Note:            None
 *******************************************************************/
void ProcessIO(void)
{   char   buf[20];
	//Action Codes:
	//0x80 Toggle Led
	//0x77 Read Data
	//0x78 Read Message Command
	

	unsigned int counter;
    //Blink the LEDs according to the USB device status
    //if(blinkStatusValid)
    //{
    //    BlinkUSBStatus();
    //}

    // User Application USB tasks
    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1)) return;
    
    if(!HIDRxHandleBusy(USBOutHandle))				//Check if data was received from the host.
    {   
	
        switch(ReceivedDataBuffer[0])				//Look at the data the host sent, to see what kind of application specific command it sent.
        {
			

            case 0x80:  //Toggle LEDs command
		        blinkStatusValid = FALSE;			//Stop blinking the LEDs automatically, going to manually control them now.
                if(mGetLED_1() == mGetLED_2())
                {
                    mLED_1_Toggle();
                    mLED_2_Toggle();
                }
                else
                {
                    if(mGetLED_1())
                    {
                        mLED_2_On();
                    }
                    else
                    {
                        mLED_2_Off();
                    }
                }
                break;



		case 0x83:	//Read Data command.  
                {
                    WORD_VAL w;
	                if(!HIDTxHandleBusy(USBInHandle))
	                {
						ToSendDataBuffer[0] = 0x83;  	//Echo back to the host the command we are fulfilling in the first byte.
	                    ToSendDataBuffer[1]=ACCEL_XOUT_L;
	 					ToSendDataBuffer[2]=ACCEL_XOUT_H;
					 	ToSendDataBuffer[3]=ACCEL_YOUT_L;
	 					ToSendDataBuffer[4]=ACCEL_YOUT_H;
	 					ToSendDataBuffer[5]=ACCEL_ZOUT_L;
	 					ToSendDataBuffer[6]=ACCEL_ZOUT_H;
						 ToSendDataBuffer[7]= GYRO_XOUT_L;
	 ToSendDataBuffer[8]= GYRO_XOUT_H;
	 ToSendDataBuffer[9]= GYRO_YOUT_L;
	 ToSendDataBuffer[10]= GYRO_YOUT_H;
	 ToSendDataBuffer[11]= GYRO_ZOUT_L;
	 ToSendDataBuffer[12]= GYRO_ZOUT_H;
					



						USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
	                }					
                }
                break;
     

	case 0x77:	//Read Data command.  
                {
                    WORD_VAL w;
	                if(!HIDTxHandleBusy(USBInHandle))
	                {
						ToSendDataBuffer[0] = 0x77;  	//Echo back to the host the command we are fulfilling in the first byte.  In this case, the Read POT (analog voltage) command.
						ToSendDataBuffer[1] = RPM.v[0];  	//RPM LSB here we willput the read data frommotion sensor command
						ToSendDataBuffer[2] = RPM.v[1];  	//RPM MSB
	                    USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
	                }					
                }
                break;
     








  
	case 0x78:	//Read Message command.  
                {
                    WORD_VAL w;
	                if(!HIDTxHandleBusy(USBInHandle))
	                {
						unsigned int counter;
						unsigned int NrOfCharsToSend=0;
						unsigned char PresentC;
						ToSendDataBuffer[0] = 0x78;  	//Echo back to the host the command we are fulfilling in the first byte.  In this case, the Read POT (analog voltage) command.
						
						if (MessageCounter<58)
							{NrOfCharsToSend=MessageCounter;}
							else
							NrOfCharsToSend=58;
						
						ToSendDataBuffer[1] = (unsigned char) NrOfCharsToSend; 

						for(counter=0;counter<NrOfCharsToSend;counter++)
							ToSendDataBuffer[2+counter] = MessageBuffer[counter].u_value;  	
						
						for(counter=58;counter<MAX_Message;counter++)
							MessageBuffer[counter-58].u_value = MessageBuffer[counter].u_value;
						MessageCounter=MessageCounter-NrOfCharsToSend;
					
						};
	
						USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
	                }					
                break;
	 

          
        }
        //Re-arm the OUT endpoint for the next packet
        USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
    }

    
}//end ProcessIO



/******************************************************************************
 * Function:        Buffering of ADC, Time Stamp and Status
 *
 *****************************************************************************/








/******************************************************************************
 * Function:        WORD_VAL ReadPOT(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          WORD_VAL - the 10-bit right justified POT value
 *
 * Side Effects:    ADC buffer value updated
 *
 * Overview:        This function reads the POT and leaves the value in the 
 *                  ADC buffer register
 *
 * Note:            None
 *****************************************************************************/

void ServiceADC( void )
{
	unsigned int offset;
	offset=8* ((~ReadActiveBufferADC10() & 0x01));
	LightBarrierADC=ReadADC10(offset);
	PressureFeedbackADC=ReadADC10(offset+1);
};

void InitADC( void )
{
	//Light Barrier AN2 
	//Valve Feedback AN3    

	CloseADC10();
	#define PARAM1 ADC_MODULE_ON | ADC_FORMAT_INTG | ADC_CLK_AUTO | ADC_AUTO_SAMPLING_ON
	#define PARAM2 ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_OFF| ADC_SAMPLES_PER_INT_2 | ADC_ALT_BUF_ON | ADC_ALT_INPUT_ON
	#define PARAM3 ADC_CONV_CLK_SYSTEM | ADC_CONV_CLK_8Tcy | ADC_SAMPLE_TIME_12
	#define PARAM4 SKIP_SCAN_ALL	
	#define PARAM5 ENABLE_AN2_ANA | ENABLE_AN3_ANA

	SetChanADC10 ( ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEA_AN2 | 
				 ADC_CH0_NEG_SAMPLEB_NVREF | ADC_CH0_POS_SAMPLEB_AN3); 	

	OpenADC10 ( PARAM1, PARAM2, PARAM3, PARAM4, PARAM5);

	EnableADC10();

   return;
}






/********************************************************************
 * Function:        void BlinkUSBStatus(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        BlinkUSBStatus turns on and off LEDs 
 *                  corresponding to the USB device state.
 *
 * Note:            mLED macros can be found in HardwareProfile.h
 *                  USBDeviceState is declared and updated in
 *                  usb_device.c.
 *******************************************************************/
void BlinkUSBStatus(void)
{
#if defined(PIC24FJ256DA210_DEV_BOARD)
    // No need to clear UIRbits.SOFIF to 0 here.
    // Callback caller is already doing that.
    #define BLINK_INTERVAL 20000
    #define BLANK_INTERVAL 200000

    static WORD blink_count=0;
    static DWORD loop_count = 0;
    
    if(loop_count == 0)
    {
        if(blink_count != 0)
        {
            loop_count = BLINK_INTERVAL;
            if(mGetLED_1())
            {
                mLED_1_Off();
                blink_count--;
            }
            else
            {
                mLED_1_On();
            }
        }
        else
        {
            loop_count = BLANK_INTERVAL;
            switch(USBDeviceState)
            {
                case ATTACHED_STATE:
                    blink_count = 1;
                    break;
                case POWERED_STATE:
                    blink_count = 2;
                    break;
                case DEFAULT_STATE:
                    blink_count = 3;
                    break;
                case ADR_PENDING_STATE:
                    blink_count = 4;
                    break;
                case ADDRESS_STATE:
                    blink_count = 5;
                    break;
                case CONFIGURED_STATE:
                    blink_count = 6;
                    break;
                case DETACHED_STATE:
                    //fall through
                default:
                    blink_count = 0;
                    break;
            }
        }
    }
    else
    {
        loop_count--;
    }

#else
    // No need to clear UIRbits.SOFIF to 0 here.
    // Callback caller is already doing that.
    static WORD led_count=0;
    
    if(led_count == 0)led_count = 10000U;
    led_count--;

    #define mLED_Both_Off()         {mLED_1_Off();mLED_2_Off();}
    #define mLED_Both_On()          {mLED_1_On();mLED_2_On();}
    #define mLED_Only_1_On()        {mLED_1_On();mLED_2_Off();}
    #define mLED_Only_2_On()        {mLED_1_Off();mLED_2_On();}

    if(USBSuspendControl == 1)
    {
        if(led_count==0)
        {
            mLED_1_Toggle();
            if(mGetLED_1())
            {
                mLED_2_On();
            }
            else
            {
                mLED_2_Off();
            }
        }//end if
    }
    else
    {
        if(USBDeviceState == DETACHED_STATE)
        {
            mLED_Both_Off();
        }
        else if(USBDeviceState == ATTACHED_STATE)
        {
            mLED_Both_On();
        }
        else if(USBDeviceState == POWERED_STATE)
        {
            mLED_Only_1_On();
        }
        else if(USBDeviceState == DEFAULT_STATE)
        {
            mLED_Only_2_On();
        }
        else if(USBDeviceState == ADDRESS_STATE)
        {
            if(led_count == 0)
            {
                mLED_1_Toggle();
                mLED_2_Off();
            }//end if
        }
        else if(USBDeviceState == CONFIGURED_STATE)
        {
            if(led_count==0)
            {
                mLED_1_Toggle();
                if(mGetLED_1())
                {
                    mLED_2_Off();
                }
                else
                {
                    mLED_2_On();
                }
            }//end if
        }
    }
#endif
}//end BlinkUSBStatus

















// ******************************************************************************************************
// ************** USB Callback Functions ****************************************************************
// ******************************************************************************************************
// The USB firmware stack will call the callback functions USBCBxxx() in response to certain USB related
// events.  For example, if the host PC is powering down, it will stop sending out Start of Frame (SOF)
// packets to your device.  In response to this, all USB devices are supposed to decrease their power
// consumption from the USB Vbus to <2.5mA each.  The USB module detects this condition (which according
// to the USB specifications is 3+ms of no bus activity/SOF packets) and then calls the USBCBSuspend()
// function.  You should modify these callback functions to take appropriate actions for each of these
// conditions.  For example, in the USBCBSuspend(), you may wish to add code that will decrease power
// consumption from Vbus to <2.5mA (such as by clock switching, turning off LEDs, putting the
// microcontroller to sleep, etc.).  Then, in the USBCBWakeFromSuspend() function, you may then wish to
// add code that undoes the power saving things done in the USBCBSuspend() function.

// The USBCBSendResume() function is special, in that the USB stack will not automatically call this
// function.  This function is meant to be called from the application firmware instead.  See the
// additional comments near the function.

/******************************************************************************
 * Function:        void USBCBSuspend(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Call back that is invoked when a USB suspend is detected
 *
 * Note:            None
 *****************************************************************************/
void USBCBSuspend(void)
{
	//Example power saving code.  Insert appropriate code here for the desired
	//application behavior.  If the microcontroller will be put to sleep, a
	//process similar to that shown below may be used:
	
	//ConfigureIOPinsForLowPower();
	//SaveStateOfAllInterruptEnableBits();
	//DisableAllInterruptEnableBits();
	//EnableOnlyTheInterruptsWhichWillBeUsedToWakeTheMicro();	//should enable at least USBActivityIF as a wake source
	//Sleep();
	//RestoreStateOfAllPreviouslySavedInterruptEnableBits();	//Preferrably, this should be done in the USBCBWakeFromSuspend() function instead.
	//RestoreIOPinsToNormal();									//Preferrably, this should be done in the USBCBWakeFromSuspend() function instead.

	//IMPORTANT NOTE: Do not clear the USBActivityIF (ACTVIF) bit here.  This bit is 
	//cleared inside the usb_device.c file.  Clearing USBActivityIF here will cause 
	//things to not work as intended.	
	

    #if defined(__C30__)
        //This function requires that the _IPL level be something other than 0.
        //  We can set it here to something other than 
        #ifndef DSPIC33E_USB_STARTER_KIT
        _IPL = 1;
        USBSleepOnSuspend();
        #endif
    #endif
}


/******************************************************************************
 * Function:        void _USB1Interrupt(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called when the USB interrupt bit is set
 *					In this example the interrupt is only used when the device
 *					goes to sleep when it receives a USB suspend command
 *
 * Note:            None
 *****************************************************************************/
#if 0
void __attribute__ ((interrupt)) _USB1Interrupt(void)
{
    #if !defined(self_powered)
        if(U1OTGIRbits.ACTVIF)
        {
            IEC5bits.USB1IE = 0;
            U1OTGIEbits.ACTVIE = 0;
            IFS5bits.USB1IF = 0;
        
            //USBClearInterruptFlag(USBActivityIFReg,USBActivityIFBitNum);
            USBClearInterruptFlag(USBIdleIFReg,USBIdleIFBitNum);
            //USBSuspendControl = 0;
        }
    #endif
}
#endif

/******************************************************************************
 * Function:        void USBCBWakeFromSuspend(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The host may put USB peripheral devices in low power
 *					suspend mode (by "sending" 3+ms of idle).  Once in suspend
 *					mode, the host may wake the device back up by sending non-
 *					idle state signalling.
 *					
 *					This call back is invoked when a wakeup from USB suspend 
 *					is detected.
 *
 * Note:            None
 *****************************************************************************/
void USBCBWakeFromSuspend(void)
{
	// If clock switching or other power savings measures were taken when
	// executing the USBCBSuspend() function, now would be a good time to
	// switch back to normal full power run mode conditions.  The host allows
	// a few milliseconds of wakeup time, after which the device must be 
	// fully back to normal, and capable of receiving and processing USB
	// packets.  In order to do this, the USB module must receive proper
	// clocking (IE: 48MHz clock must be available to SIE for full speed USB
	// operation).
}

/********************************************************************
 * Function:        void USBCB_SOF_Handler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The USB host sends out a SOF packet to full-speed
 *                  devices every 1 ms. This interrupt may be useful
 *                  for isochronous pipes. End designers should
 *                  implement callback routine as necessary.
 *
 * Note:            None
 *******************************************************************/
void USBCB_SOF_Handler(void)
{
    // No need to clear UIRbits.SOFIF to 0 here.
    // Callback caller is already doing that.
}

/*******************************************************************
 * Function:        void USBCBErrorHandler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The purpose of this callback is mainly for
 *                  debugging during development. Check UEIR to see
 *                  which error causes the interrupt.
 *
 * Note:            None
 *******************************************************************/
void USBCBErrorHandler(void)
{
    // No need to clear UEIR to 0 here.
    // Callback caller is already doing that.

	// Typically, user firmware does not need to do anything special
	// if a USB error occurs.  For example, if the host sends an OUT
	// packet to your device, but the packet gets corrupted (ex:
	// because of a bad connection, or the user unplugs the
	// USB cable during the transmission) this will typically set
	// one or more USB error interrupt flags.  Nothing specific
	// needs to be done however, since the SIE will automatically
	// send a "NAK" packet to the host.  In response to this, the
	// host will normally retry to send the packet again, and no
	// data loss occurs.  The system will typically recover
	// automatically, without the need for application firmware
	// intervention.
	
	// Nevertheless, this callback function is provided, such as
	// for debugging purposes.
}


/*******************************************************************
 * Function:        void USBCBCheckOtherReq(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        When SETUP packets arrive from the host, some
 * 					firmware must process the request and respond
 *					appropriately to fulfill the request.  Some of
 *					the SETUP packets will be for standard
 *					USB "chapter 9" (as in, fulfilling chapter 9 of
 *					the official USB specifications) requests, while
 *					others may be specific to the USB device class
 *					that is being implemented.  For example, a HID
 *					class device needs to be able to respond to
 *					"GET REPORT" type of requests.  This
 *					is not a standard USB chapter 9 request, and 
 *					therefore not handled by usb_device.c.  Instead
 *					this request should be handled by class specific 
 *					firmware, such as that contained in usb_function_hid.c.
 *
 * Note:            None
 *******************************************************************/
void USBCBCheckOtherReq(void)
{
    USBCheckHIDRequest();
}//end


/*******************************************************************
 * Function:        void USBCBStdSetDscHandler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The USBCBStdSetDscHandler() callback function is
 *					called when a SETUP, bRequest: SET_DESCRIPTOR request
 *					arrives.  Typically SET_DESCRIPTOR requests are
 *					not used in most applications, and it is
 *					optional to support this type of request.
 *
 * Note:            None
 *******************************************************************/
void USBCBStdSetDscHandler(void)
{
    // Must claim session ownership if supporting this request
}//end


/*******************************************************************
 * Function:        void USBCBInitEP(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called when the device becomes
 *                  initialized, which occurs after the host sends a
 * 					SET_CONFIGURATION (wValue not = 0) request.  This 
 *					callback function should initialize the endpoints 
 *					for the device's usage according to the current 
 *					configuration.
 *
 * Note:            None
 *******************************************************************/
void USBCBInitEP(void)
{
    //enable the HID endpoint
    USBEnableEndpoint(HID_EP,USB_IN_ENABLED|USB_OUT_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
    //Re-arm the OUT endpoint for the next packet
    USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
}

/********************************************************************
 * Function:        void USBCBSendResume(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The USB specifications allow some types of USB
 * 					peripheral devices to wake up a host PC (such
 *					as if it is in a low power suspend to RAM state).
 *					This can be a very useful feature in some
 *					USB applications, such as an Infrared remote
 *					control	receiver.  If a user presses the "power"
 *					button on a remote control, it is nice that the
 *					IR receiver can detect this signalling, and then
 *					send a USB "command" to the PC to wake up.
 *					
 *					The USBCBSendResume() "callback" function is used
 *					to send this special USB signalling which wakes 
 *					up the PC.  This function may be called by
 *					application firmware to wake up the PC.  This
 *					function will only be able to wake up the host if
 *                  all of the below are true:
 *					
 *					1.  The USB driver used on the host PC supports
 *						the remote wakeup capability.
 *					2.  The USB configuration descriptor indicates
 *						the device is remote wakeup capable in the
 *						bmAttributes field.
 *					3.  The USB host PC is currently sleeping,
 *						and has previously sent your device a SET 
 *						FEATURE setup packet which "armed" the
 *						remote wakeup capability.   
 *
 *                  If the host has not armed the device to perform remote wakeup,
 *                  then this function will return without actually performing a
 *                  remote wakeup sequence.  This is the required behavior, 
 *                  as a USB device that has not been armed to perform remote 
 *                  wakeup must not drive remote wakeup signalling onto the bus;
 *                  doing so will cause USB compliance testing failure.
 *                  
 *					This callback should send a RESUME signal that
 *                  has the period of 1-15ms.
 *
 * Note:            This function does nothing and returns quickly, if the USB
 *                  bus and host are not in a suspended condition, or are 
 *                  otherwise not in a remote wakeup ready state.  Therefore, it
 *                  is safe to optionally call this function regularly, ex: 
 *                  anytime application stimulus occurs, as the function will
 *                  have no effect, until the bus really is in a state ready
 *                  to accept remote wakeup. 
 *
 *                  When this function executes, it may perform clock switching,
 *                  depending upon the application specific code in 
 *                  USBCBWakeFromSuspend().  This is needed, since the USB
 *                  bus will no longer be suspended by the time this function
 *                  returns.  Therefore, the USB module will need to be ready
 *                  to receive traffic from the host.
 *
 *                  The modifiable section in this routine may be changed
 *                  to meet the application needs. Current implementation
 *                  temporary blocks other functions from executing for a
 *                  period of ~3-15 ms depending on the core frequency.
 *
 *                  According to USB 2.0 specification section 7.1.7.7,
 *                  "The remote wakeup device must hold the resume signaling
 *                  for at least 1 ms but for no more than 15 ms."
 *                  The idea here is to use a delay counter loop, using a
 *                  common value that would work over a wide range of core
 *                  frequencies.
 *                  That value selected is 1800. See table below:
 *                  ==========================================================
 *                  Core Freq(MHz)      MIP         RESUME Signal Period (ms)
 *                  ==========================================================
 *                      48              12          1.05
 *                       4              1           12.6
 *                  ==========================================================
 *                  * These timing could be incorrect when using code
 *                    optimization or extended instruction mode,
 *                    or when having other interrupts enabled.
 *                    Make sure to verify using the MPLAB SIM's Stopwatch
 *                    and verify the actual signal on an oscilloscope.
 *******************************************************************/
void USBCBSendResume(void)
{
    static WORD delay_count;
    
    //First verify that the host has armed us to perform remote wakeup.
    //It does this by sending a SET_FEATURE request to enable remote wakeup,
    //usually just before the host goes to standby mode (note: it will only
    //send this SET_FEATURE request if the configuration descriptor declares
    //the device as remote wakeup capable, AND, if the feature is enabled
    //on the host (ex: on Windows based hosts, in the device manager 
    //properties page for the USB device, power management tab, the 
    //"Allow this device to bring the computer out of standby." checkbox 
    //should be checked).
    if(USBGetRemoteWakeupStatus() == TRUE) 
    {
        //Verify that the USB bus is in fact suspended, before we send
        //remote wakeup signalling.
        if(USBIsBusSuspended() == TRUE)
        {
            USBMaskInterrupts();
            
            //Clock switch to settings consistent with normal USB operation.
            USBCBWakeFromSuspend();
            USBSuspendControl = 0; 
            USBBusIsSuspended = FALSE;  //So we don't execute this code again, 
                                        //until a new suspend condition is detected.

            //Section 7.1.7.7 of the USB 2.0 specifications indicates a USB
            //device must continuously see 5ms+ of idle on the bus, before it sends
            //remote wakeup signalling.  One way to be certain that this parameter
            //gets met, is to add a 2ms+ blocking delay here (2ms plus at 
            //least 3ms from bus idle to USBIsBusSuspended() == TRUE, yeilds
            //5ms+ total delay since start of idle).
            delay_count = 3600U;        
            do
            {
                delay_count--;
            }while(delay_count);
            
            //Now drive the resume K-state signalling onto the USB bus.
            USBResumeControl = 1;       // Start RESUME signaling
            delay_count = 1800U;        // Set RESUME line for 1-13 ms
            do
            {
                delay_count--;
            }while(delay_count);
            USBResumeControl = 0;       //Finished driving resume signalling

            USBUnmaskInterrupts();
        }
    }
}


/*******************************************************************
 * Function:        BOOL USER_USB_CALLBACK_EVENT_HANDLER(
 *                        USB_EVENT event, void *pdata, WORD size)
 *
 * PreCondition:    None
 *
 * Input:           USB_EVENT event - the type of event
 *                  void *pdata - pointer to the event data
 *                  WORD size - size of the event data
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called from the USB stack to
 *                  notify a user application that a USB event
 *                  occured.  This callback is in interrupt context
 *                  when the USB_INTERRUPT option is selected.
 *
 * Note:            None
 *******************************************************************/
BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size)
{
    switch(event)
    {
        case EVENT_TRANSFER:
            //Add application specific callback task or callback function here if desired.
            break;
        case EVENT_SOF:
            USBCB_SOF_Handler();
            break;
        case EVENT_SUSPEND:
            USBCBSuspend();
            break;
        case EVENT_RESUME:
            USBCBWakeFromSuspend();
            break;
        case EVENT_CONFIGURED: 
            USBCBInitEP();
            break;
        case EVENT_SET_DESCRIPTOR:
            USBCBStdSetDscHandler();
            break;
        case EVENT_EP0_REQUEST:
            USBCBCheckOtherReq();
            break;
        case EVENT_BUS_ERROR:
            USBCBErrorHandler();
            break;
        case EVENT_TRANSFER_TERMINATED:
            //Add application specific callback task or callback function here if desired.
            //The EVENT_TRANSFER_TERMINATED event occurs when the host performs a CLEAR
            //FEATURE (endpoint halt) request on an application endpoint which was 
            //previously armed (UOWN was = 1).  Here would be a good place to:
            //1.  Determine which endpoint the transaction that just got terminated was 
            //      on, by checking the handle value in the *pdata.
            //2.  Re-arm the endpoint if desired (typically would be the case for OUT 
            //      endpoints).
            break;
        default:
            break;
    }      
    return TRUE; 
}

/** EOF main.c *************************************************/
#endif
